# header: importing useful modules and functions

import csv
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import fmin

#%%#########################################################
# PART 2: Analysis of IPCC/CMIP5 Earth system model results
############################################################

#============
# SECTION 1.1
#============

# ancillary lists for loading data
model_list = ['ACCESS-10','ACCESS-13','BCC-CSM-11','BCC-CSM-11m','CanESM2','CCSM4','CNRM-CM5','CNRM-CM5-2','CSIRO-Mk360','GFDL-CM3','GFDL-ESM2G','GFDL-ESM2M','GISS-E2-H','GISS-E2-R','HadGEM2-ES','IPSL-CM5A-LR','IPSL-CM5A-MR','IPSL-CM5B-LR','MIROC5','MIROC-ESM','MPI-ESM-LR','MPI-ESM-MR','MPI-ESM-P','MRI-CGCM3','NorESM1-M']
model_length = {'ACCESS-10':150,'ACCESS-13':151,'BCC-CSM-11':150,'BCC-CSM-11m':150,'CanESM2':150,'CCSM4':151,'CNRM-CM5':150,'CNRM-CM5-2':140,'CSIRO-Mk360':150,'GFDL-CM3':150,'GFDL-ESM2G':300,'GFDL-ESM2M':300,'GISS-E2-H':151,'GISS-E2-R':151,'HadGEM2-ES':151,'IPSL-CM5A-LR':260,'IPSL-CM5A-MR':140,'IPSL-CM5B-LR':160,'MIROC5':151,'MIROC-ESM':150,'MPI-ESM-LR':150,'MPI-ESM-MR':150,'MPI-ESM-P':150,'MRI-CGCM3':150,'NorESM1-M':150}

# create array of final variables
# tas in {degC}; rsdt,rlut,rsut in {W/m2}
for var in ['tas','rsdt','rsut','rlut']:
#    exec(var+'_ctrl = np.zeros((np.max(model_length.values()),len(model_list))) * np.nan')
#    exec(var+'_quad = np.zeros((np.max(model_length.values()),len(model_list))) * np.nan')
    exec(var+'_ctrl = np.zeros((300,len(model_list))) * np.nan') #-> control simulation with preindustrial atmCO2 = 284ppm
    exec(var+'_quad = np.zeros((300,len(model_list))) * np.nan') #-> 4xCO2 simulation atmCO2=1136ppm

# load pre-processed CMIP5 results
for mod in model_list:
    for sim in ['ctrl','quad']:
        TMP = np.array([line for line in csv.reader(open('data/Climate_CMIP5/Climate_'+mod+'.'+sim+'_global.csv','r'))][1:],dtype=float)
        lgd = [line for line in csv.reader(open('data/Climate_CMIP5/Climate_'+mod+'.'+sim+'_global.csv','r'))][0]
        for var in ['tas','rsdt','rsut','rlut']:
            exec(var+'_'+sim+'[:len(TMP),model_list.index(mod)] = TMP[:,lgd.index(var)]')


# create net radiative budget variable
netrad_ctrl = rsdt_ctrl - rsut_ctrl - rlut_ctrl
netrad_quad = rsdt_quad - rsut_quad - rlut_quad

# Compute Radiative Imbalance and Warming
netImbalance = netrad_quad - netrad_ctrl
Warming      = tas_quad- tas_ctrl

#%%==========
# SECTION 1.2
#============

# fit equilibrium temperature change
# the function fmin find the couple of solutin coeff and T4x that minimize the error of the linear fit
coeff = np.zeros(len(model_list))
T4x = np.zeros(len(model_list))
for mod in model_list:
    m = model_list.index(mod)
    length = model_length[mod] #!!!
    def err(var):
        D_T = var[0] * (netImbalance[:length,m]) + var[1]
        return sum(( D_T - (Warming)[:length,m] )**2)
    [coeff[m],T4x[m]] = fmin(err,[-1.,3.],disp=False)


# deduce climate sensitivity
climsens = T4x / (5.35*np.log(4))

#============
# SECTION 2.1
#============
# Question 1
# fit dynamical parameters
a_T1 = np.zeros(len(model_list))
tau_T1 = np.zeros(len(model_list))
tau_T2 = np.zeros(len(model_list))
for mod in model_list:
    m = model_list.index(mod)
    length = model_length[mod] #!!!
    time = np.arange(0,length)+0.5
    def err(var):
        D_T = T4x[m] * (1 - var[0]*np.exp(-time/var[1]) - (1-var[0])*np.exp(-time/var[2]))
        return sum(( D_T - (tas_quad-tas_ctrl)[:length,m] )**2)
    [a_T1[m],tau_T1[m],tau_T2[m]] = fmin(err,[0.5,2,200],disp=False)
a_T2 = 1-a_T1

### Fig. 8 ###
# plot fit of climate response
plt.figure()
for mod in model_list:
    m = model_list.index(mod)
    plt.subplot(5,5,m+1)
    plt.plot((tas_quad-tas_ctrl)[:,m],marker='+',ls='none')    
    plt.plot(T4x[m]*(1 - a_T1[m]*np.exp(-np.arange(250)/tau_T1[m]) - a_T2[m]*np.exp(-np.arange(250)/tau_T2[m])),lw=2)
    plt.title(mod)
 

